/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//Copyright (c) 2010 by disy Informationssysteme GmbH
package net.disy.legato.testing.server;

import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.text.MessageFormat;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

// NOT_PUBLISHED
public class PortFinder {

  public static final int AUTO_PORT_RANGE_START = 7000;
  public static final int AUTO_PORT_RANGE_END = 9999;

  private static final Log logger = LogFactory.getLog(PortFinder.class);

  public static int findFreePort(final String host) {
    int freePort = -1;
    final long start = System.currentTimeMillis();
    for (int portToTry = AUTO_PORT_RANGE_START; portToTry <= AUTO_PORT_RANGE_END; portToTry++) {
      Socket socket = null;
      try {
        logger
            .debug(MessageFormat
                .format(
                    "Trying to open the socket [{0}:{1,number,####}] ({2} ms).", host, portToTry, System.currentTimeMillis() - start)); //$NON-NLS-1$
        socket = new Socket(host, portToTry);
        logger
            .debug(MessageFormat
                .format(
                    "Socket [{0}:{1,number,####}] is opened ({2} ms.)", host, portToTry, System.currentTimeMillis() - start)); //$NON-NLS-1$
      }
      catch (final UnknownHostException uhex) {
        throw new ExceptionInInitializerError(uhex);
      }
      catch (final IOException ioex) {
        logger
            .debug(MessageFormat
                .format(
                    "Socket [{0}:{1,number,####}] could not be opened, the port seems to be free ({2} ms.)", host, portToTry, System.currentTimeMillis() - start)); //$NON-NLS-1$
        // No server at this port
        freePort = portToTry;
        break;
      }
      finally {
        if (socket != null) {
          try {
            logger.debug(MessageFormat.format(
                "Closing the socket ({0} ms.)", System.currentTimeMillis() - start)); //$NON-NLS-1$
            socket.close();
            logger.debug(MessageFormat.format(
                "The socket is closed ({0} ms.)", System.currentTimeMillis() - start)); //$NON-NLS-1$
          }
          catch (final IOException ioex) {
            throw new ExceptionInInitializerError(ioex);
          }
        }
      }
    }
    if (freePort == -1) {
      throw new IllegalArgumentException(
          MessageFormat
              .format(
                  "Could not find a free port in range [{0}, {1}].", AUTO_PORT_RANGE_START, AUTO_PORT_RANGE_END)); //$NON-NLS-1$
    }
    return freePort;
  }

}
